@@ -9,11 +9,11 @@ use crossbeam_epoch::Guard;
 use crate::{
     child_ptr::NodePtr,
     lock::{ConcreteReadGuard, ReadGuard},
-    node_16::Node16,
-    node_256::Node256,
-    node_4::Node4,
-    node_48::Node48,
-    utils::convert_type_to_version,
+    node_16::{Node16, Node16Iter},
+    node_256::{Node256, Node256Iter},
+    node_4::{Node4, Node4Iter},
+    node_48::{Node48, Node48Iter},
+    utils::{convert_type_to_version, ArtError},
 };
 
 pub(crate) const MAX_STORED_PREFIX_LEN: usize = 10;
@@ -63,12 +63,32 @@ pub(crate) trait Node {
     fn insert(&mut self, key: u8, node: NodePtr);
     fn change(&mut self, key: u8, val: NodePtr);
     fn get_child(&self, key: u8) -> Option<NodePtr>;
-    fn get_children(&self, start: u8, end: u8) -> Vec<(u8, NodePtr)>;
+    fn get_children(&self, start: u8, end: u8) -> NodeIter<'_>;
     fn remove(&mut self, k: u8);
     fn copy_to<N: Node>(&self, dst: &mut N);
     fn get_type() -> NodeType;
 }
 
+pub(crate) enum NodeIter<'a> {
+    N4(Node4Iter<'a>),
+    N16(Node16Iter<'a>),
+    N48(Node48Iter<'a>),
+    N256(Node256Iter<'a>),
+}
+
+impl Iterator for NodeIter<'_> {
+    type Item = (u8, NodePtr);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        match self {
+            NodeIter::N4(iter) => iter.next(),
+            NodeIter::N16(iter) => iter.next(),
+            NodeIter::N48(iter) => iter.next(),
+            NodeIter::N256(iter) => iter.next(),
+        }
+    }
+}
+
 #[repr(C)]
 pub(crate) struct BaseNode {
     // 2b type | 60b version | 1b lock | 1b obsolete
@@ -142,7 +162,7 @@ macro_rules! gen_method_mut {
 }
 
 gen_method!(get_child, (k: u8), Option<NodePtr>);
-gen_method!(get_children, (start: u8, end: u8), Vec<(u8, NodePtr)>);
+gen_method!(get_children, (start: u8, end: u8), NodeIter<'_>);
 gen_method_mut!(change, (key: u8, val: NodePtr), ());
 gen_method_mut!(remove, (key: u8), ());
 
@@ -198,10 +218,10 @@ impl BaseNode {
         }
     }
 
-    pub(crate) fn read_lock(&self) -> Result<ReadGuard, usize> {
+    pub(crate) fn read_lock(&self) -> Result<ReadGuard, ArtError> {
         let version = self.type_version_lock_obsolete.load(Ordering::Acquire);
         if Self::is_locked(version) || Self::is_obsolete(version) {
-            return Err(version);
+            return Err(ArtError::Locked(version));
         }
 
         Ok(ReadGuard::new(version, self))
@@ -235,13 +255,13 @@ impl BaseNode {
         key: u8,
         val: NodePtr,
         guard: &Guard,
-    ) -> Result<(), ()> {
+    ) -> Result<(), ArtError> {
         if !n.as_ref().is_full() {
             if let Some(p) = parent_node {
-                p.unlock().map_err(|_| ())?;
+                p.unlock()?;
             }
 
-            let mut write_n = n.upgrade_to_write_lock().map_err(|_| ())?;
+            let mut write_n = n.upgrade_to_write_lock().map_err(|v| v.1)?;
 
             write_n.as_mut().insert(key, val);
             return Ok(());
@@ -249,9 +269,9 @@ impl BaseNode {
 
         let p = parent_node.expect("parent node must present when current node is full");
 
-        let mut write_p = p.upgrade().map_err(|_| ())?;
+        let mut write_p = p.upgrade().map_err(|v| v.1)?;
 
-        let mut write_n = n.upgrade_to_write_lock().map_err(|_| ())?;
+        let mut write_n = n.upgrade_to_write_lock().map_err(|v| v.1)?;
 
         let mut n_big = BaseNode::make_node::<BiggerT>(write_n.as_ref().base().prefix());
         write_n.as_ref().copy_to(n_big.as_mut());
@@ -278,7 +298,7 @@ impl BaseNode {
         key: u8,
         val: NodePtr,
         guard: &Guard,
-    ) -> Result<(), ()> {
+    ) -> Result<(), ArtError> {
         match node.as_ref().get_type() {
             NodeType::N4 => Self::insert_grow::<Node4, Node16>(
                 node.into_concrete(),
