@@ -1,5 +1,5 @@
 use crate::{
-    base_node::{BaseNode, Node, NodeType},
+    base_node::{BaseNode, Node, NodeIter, NodeType},
     child_ptr::NodePtr,
 };
 
@@ -95,19 +95,40 @@ impl Node16 {
     }
 }
 
+pub(crate) struct Node16Iter<'a> {
+    node: &'a Node16,
+    start_pos: usize,
+    end_pos: usize,
+}
+
+impl<'a> Iterator for Node16Iter<'a> {
+    type Item = (u8, NodePtr);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        if self.start_pos > self.end_pos {
+            return None;
+        }
+        let key = Node16::flip_sign(self.node.keys[self.start_pos]);
+        let child = self.node.children[self.start_pos];
+        self.start_pos += 1;
+        Some((key, child))
+    }
+}
+
 impl Node for Node16 {
     fn get_type() -> NodeType {
         NodeType::N16
     }
 
-    fn get_children(&self, start: u8, end: u8) -> Vec<(u8, NodePtr)> {
+    fn get_children(&self, start: u8, end: u8) -> NodeIter {
         if self.base.count == 0 {
             // FIXME: the node may be empty due to deletion, this is not intended, we should fix the delete logic
-            return vec![];
+            return NodeIter::N16(Node16Iter {
+                node: self,
+                start_pos: 1,
+                end_pos: 0,
+            });
         }
-
-        let mut children = Vec::with_capacity(16);
-
         let start_pos = self.get_child_pos(start).unwrap_or(0);
         let end_pos = self
             .get_child_pos(end)
@@ -115,11 +136,11 @@ impl Node for Node16 {
 
         debug_assert!(end_pos < 16);
 
-        for i in start_pos..=end_pos {
-            children.push((Self::flip_sign(self.keys[i]), self.children[i]));
-        }
-
-        children
+        NodeIter::N16(Node16Iter {
+            node: self,
+            start_pos,
+            end_pos,
+        })
     }
 
     fn remove(&mut self, k: u8) {
