@@ -1,5 +1,5 @@
 use crate::{
-    base_node::{BaseNode, Node, NodeType},
+    base_node::{BaseNode, Node, NodeIter, NodeType},
     child_ptr::NodePtr,
 };
 
@@ -10,27 +10,46 @@ pub(crate) struct Node256 {
     children: [NodePtr; 256],
 }
 
+pub(crate) struct Node256Iter<'a> {
+    start: u8,
+    idx: u16,
+    iter_children: std::slice::Iter<'a, NodePtr>,
+}
+
+impl<'a> Iterator for Node256Iter<'a> {
+    type Item = (u8, NodePtr);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        loop {
+            let child = self.iter_children.next()?;
+            let cur = self.idx;
+            self.idx += 1;
+            if child.is_null() {
+                continue;
+            } else {
+                return Some((self.start + cur as u8, *child));
+            }
+        }
+    }
+}
+
 impl Node for Node256 {
     fn get_type() -> NodeType {
         NodeType::N256
     }
 
-    fn get_children(&self, start: u8, end: u8) -> Vec<(u8, NodePtr)> {
-        let mut children = Vec::with_capacity(48);
-
-        for i in start..=end {
-            if !self.children[i as usize].is_null() {
-                children.push((i, self.children[i as usize]));
-            }
-        }
-
-        children
+    fn get_children(&self, start: u8, end: u8) -> NodeIter {
+        NodeIter::N256(Node256Iter {
+            start,
+            idx: 0,
+            iter_children: self.children[start as usize..=end as usize].iter(),
+        })
     }
 
     fn copy_to<N: Node>(&self, dst: &mut N) {
-        for i in 0..256 {
-            if !self.children[i].is_null() {
-                dst.insert(i as u8, self.children[i]);
+        for (i, c) in self.children.iter().enumerate() {
+            if !c.is_null() {
+                dst.insert(i as u8, *c);
             }
         }
     }
