@@ -1,5 +1,5 @@
 use crate::{
-    base_node::{BaseNode, Node, NodeType},
+    base_node::{BaseNode, Node, NodeIter, NodeType},
     child_ptr::NodePtr,
 };
 
@@ -11,6 +11,33 @@ pub(crate) struct Node4 {
     children: [NodePtr; 4],
 }
 
+pub(crate) struct Node4Iter<'a> {
+    start: u8,
+    end: u8,
+    idx: u8,
+    cnt: u8,
+    node: &'a Node4,
+}
+
+impl Iterator for Node4Iter<'_> {
+    type Item = (u8, NodePtr);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        loop {
+            if self.idx >= self.cnt {
+                return None;
+            }
+            let cur = self.idx;
+            self.idx += 1;
+
+            let key = self.node.keys[cur as usize];
+            if key >= self.start && key <= self.end {
+                return Some((key, self.node.children[cur as usize]));
+            }
+        }
+    }
+}
+
 impl Node for Node4 {
     fn get_type() -> NodeType {
         NodeType::N4
@@ -38,16 +65,14 @@ impl Node for Node4 {
         }
     }
 
-    fn get_children(&self, start: u8, end: u8) -> Vec<(u8, NodePtr)> {
-        let mut out_children = Vec::with_capacity(4);
-
-        for i in 0..self.base.count as usize {
-            if self.keys[i] >= start && self.keys[i] <= end {
-                out_children.push((self.keys[i], self.children[i]));
-            }
-        }
-
-        out_children
+    fn get_children(&self, start: u8, end: u8) -> NodeIter {
+        NodeIter::N4(Node4Iter {
+            start,
+            end,
+            idx: 0,
+            cnt: self.base.count as u8,
+            node: self,
+        })
     }
 
     fn copy_to<N: Node>(&self, dst: &mut N) {
