@@ -1,3 +1,4 @@
+use crate::utils::ArtError;
 use crate::{
     base_node::BaseNode, child_ptr::NodePtr, key::RawKey, lock::ReadGuard, utils::KeyTracker,
 };
@@ -54,7 +55,7 @@ impl<'a, T: RawKey> RangeScan<'a, T> {
         false
     }
 
-    pub(crate) fn scan(&mut self) -> Result<usize, usize> {
+    pub(crate) fn scan(&mut self) -> Result<usize, ArtError> {
         let mut level = 0;
         let mut node: ReadGuard;
         let mut next_node = self.root;
@@ -64,7 +65,7 @@ impl<'a, T: RawKey> RangeScan<'a, T> {
 
         let mut key_tracker = KeyTracker::default();
 
-        'inner: loop {
+        loop {
             node = unsafe { &*next_node }.read_lock()?;
 
             let prefix_check_result =
@@ -93,22 +94,20 @@ impl<'a, T: RawKey> RangeScan<'a, T> {
                         let children = node.as_ref().get_children(start_level, end_level);
                         node.check_version()?;
 
-                        for (k, n) in children.iter() {
-                            key_tracker.push(*k);
-                            if *k == start_level {
-                                self.find_start(*n, level + 1, &node, key_tracker.clone())
-                                    .map_err(|_| 0_usize)?;
-                            } else if *k > start_level && *k < end_level {
-                                let cur_key = KeyTracker::append_prefix(*n, &key_tracker);
-                                self.copy_node(*n, &cur_key).map_err(|_| 0_usize)?;
-                            } else if *k == end_level {
-                                self.find_end(*n, level + 1, &node, key_tracker.clone())
-                                    .map_err(|_| 0_usize)?;
+                        for (k, n) in children {
+                            key_tracker.push(k);
+                            if k == start_level {
+                                self.find_start(n, &node, key_tracker.clone())?;
+                            } else if k > start_level && k < end_level {
+                                let cur_key = KeyTracker::append_prefix(n, &key_tracker);
+                                self.copy_node(n, &cur_key)?;
+                            } else if k == end_level {
+                                self.find_end(n, &node, key_tracker.clone())?;
                             }
                             key_tracker.pop();
 
                             if self.to_continue > 0 {
-                                break 'inner;
+                                return Ok(self.result_found);
                             }
                         }
                     } else {
@@ -121,9 +120,8 @@ impl<'a, T: RawKey> RangeScan<'a, T> {
 
                         key_tracker.push(start_level);
                         if next_node_tmp.is_leaf() {
-                            self.copy_node(next_node_tmp, &key_tracker)
-                                .map_err(|_| 0_usize)?;
-                            break;
+                            self.copy_node(next_node_tmp, &key_tracker)?;
+                            return Ok(self.result_found);
                         }
                         next_node = next_node_tmp.as_ptr();
 
@@ -131,39 +129,37 @@ impl<'a, T: RawKey> RangeScan<'a, T> {
                         parent_node = Some(node);
                         continue;
                     }
-                    break;
+                    return Ok(self.result_found);
                 }
                 PrefixCheckEqualsResult::Contained => {
-                    self.copy_node(NodePtr::from_node(node.as_ref()), &key_tracker)
-                        .map_err(|_| 0_usize)?;
+                    self.copy_node(NodePtr::from_node(node.as_ref()), &key_tracker)?;
+                    return Ok(self.result_found);
                 }
                 PrefixCheckEqualsResult::NotMatch => {
                     return Ok(0);
                 }
             }
-            break;
         }
-
-        Ok(self.result_found)
     }
 
     fn find_end(
         &mut self,
         node: NodePtr,
-        mut level: u32,
         parent_node: &ReadGuard,
         mut key_tracker: KeyTracker,
-    ) -> Result<(), ()> {
+    ) -> Result<(), ArtError> {
         if node.is_leaf() {
             return self.copy_node(node, &key_tracker);
         }
 
-        let node = unsafe { &*node.as_ptr() }.read_lock().map_err(|_| {})?;
-        let prefix_result =
-            self.check_prefix_compare(node.as_ref(), self.end, 255, &mut level, &mut key_tracker)?;
+        let mut level = key_tracker.len();
 
-        parent_node.check_version().map_err(|_| {})?;
-        node.check_version().map_err(|_| {})?;
+        let node = unsafe { &*node.as_ptr() }.read_lock()?;
+        let prefix_result =
+            self.check_prefix_compare(node.as_ref(), self.end, 255, &mut level, &mut key_tracker);
+
+        parent_node.check_version()?;
+        node.check_version()?;
 
         match prefix_result {
             cmp::Ordering::Greater => Ok(()),
@@ -175,14 +171,14 @@ impl<'a, T: RawKey> RangeScan<'a, T> {
                 };
 
                 let children = node.as_ref().get_children(0, end_level);
-                node.check_version().map_err(|_| ())?;
-                for (k, n) in children.iter() {
-                    key_tracker.push(*k);
-                    if *k == end_level {
-                        self.find_end(*n, level + 1, &node, key_tracker.clone())?;
-                    } else if *k < end_level {
-                        let cur_key = KeyTracker::append_prefix(*n, &key_tracker);
-                        self.copy_node(*n, &cur_key)?;
+                node.check_version()?;
+                for (k, n) in children {
+                    key_tracker.push(k);
+                    if k == end_level {
+                        self.find_end(n, &node, key_tracker.clone())?;
+                    } else if k < end_level {
+                        let cur_key = KeyTracker::append_prefix(n, &key_tracker);
+                        self.copy_node(n, &cur_key)?;
                     }
                     key_tracker.pop();
                     if self.to_continue != 0 {
@@ -198,20 +194,21 @@ impl<'a, T: RawKey> RangeScan<'a, T> {
     fn find_start(
         &mut self,
         node: NodePtr,
-        mut level: u32,
         parent_node: &ReadGuard,
         mut key_tracker: KeyTracker,
-    ) -> Result<(), ()> {
+    ) -> Result<(), ArtError> {
         if node.is_leaf() {
             return self.copy_node(node, &key_tracker);
         }
 
-        let node = unsafe { &*node.as_ptr() }.read_lock().map_err(|_| {})?;
-        let prefix_result =
-            self.check_prefix_compare(node.as_ref(), self.start, 0, &mut level, &mut key_tracker)?;
+        let mut level = key_tracker.len();
 
-        parent_node.check_version().map_err(|_| {})?;
-        node.check_version().map_err(|_| {})?;
+        let node = unsafe { &*node.as_ptr() }.read_lock()?;
+        let prefix_result =
+            self.check_prefix_compare(node.as_ref(), self.start, 0, &mut level, &mut key_tracker);
+
+        parent_node.check_version()?;
+        node.check_version()?;
 
         match prefix_result {
             cmp::Ordering::Greater => {
@@ -223,16 +220,17 @@ impl<'a, T: RawKey> RangeScan<'a, T> {
                 } else {
                     0
                 };
-                let children = node.as_ref().get_children(start_level, 255);
-                node.check_version().map_err(|_| ())?;
 
-                for (k, n) in children.iter() {
-                    key_tracker.push(*k);
-                    if *k == start_level {
-                        self.find_start(*n, level + 1, &node, key_tracker.clone())?;
-                    } else if *k > start_level {
-                        let cur_key = KeyTracker::append_prefix(*n, &key_tracker);
-                        self.copy_node(*n, &cur_key)?;
+                let children = node.as_ref().get_children(start_level, 255);
+                node.check_version()?;
+
+                for (k, n) in children {
+                    key_tracker.push(k);
+                    if k == start_level {
+                        self.find_start(n, &node, key_tracker.clone())?;
+                    } else if k > start_level {
+                        let cur_key = KeyTracker::append_prefix(n, &key_tracker);
+                        self.copy_node(n, &cur_key)?;
                     }
                     key_tracker.pop();
                     if self.to_continue != 0 {
@@ -245,7 +243,7 @@ impl<'a, T: RawKey> RangeScan<'a, T> {
         }
     }
 
-    fn copy_node(&mut self, node: NodePtr, key_tracker: &KeyTracker) -> Result<(), ()> {
+    fn copy_node(&mut self, node: NodePtr, key_tracker: &KeyTracker) -> Result<(), ArtError> {
         if node.is_leaf() {
             if self.key_in_range(key_tracker) {
                 if self.result_found == self.result.len() {
@@ -256,17 +254,17 @@ impl<'a, T: RawKey> RangeScan<'a, T> {
                 self.result_found += 1;
             };
         } else {
-            let node = unsafe { &*node.as_ptr() }.read_lock().map_err(|_| ())?;
+            let node = unsafe { &*node.as_ptr() }.read_lock()?;
             let mut key_tracker = key_tracker.clone();
 
             let children = node.as_ref().get_children(0, 255);
-            node.check_version().map_err(|_| ())?;
+            node.check_version()?;
 
-            for (k, c) in children.iter() {
-                key_tracker.push(*k);
+            for (k, c) in children {
+                key_tracker.push(k);
 
-                let cur_key = KeyTracker::append_prefix(*c, &key_tracker);
-                self.copy_node(*c, &cur_key)?;
+                let cur_key = KeyTracker::append_prefix(c, &key_tracker);
+                self.copy_node(c, &cur_key)?;
 
                 if self.to_continue != 0 {
                     break;
@@ -283,13 +281,13 @@ impl<'a, T: RawKey> RangeScan<'a, T> {
         n: &BaseNode,
         k: &T,
         fill_key: u8,
-        level: &mut u32,
+        level: &mut usize,
         key_tracker: &mut KeyTracker,
-    ) -> Result<cmp::Ordering, ()> {
+    ) -> cmp::Ordering {
         let n_prefix = n.prefix();
         if !n_prefix.is_empty() {
             for (i, cur_key) in n_prefix.iter().enumerate() {
-                let k_level = if k.len() as u32 > *level {
+                let k_level = if k.len() > *level {
                     k.as_bytes()[*level as usize]
                 } else {
                     fill_key
@@ -301,18 +299,18 @@ impl<'a, T: RawKey> RangeScan<'a, T> {
                     for v in n_prefix.iter().take(n_prefix.len()).skip(i + 1) {
                         key_tracker.push(*v);
                     }
-                    return Ok(cmp::Ordering::Less);
+                    return cmp::Ordering::Less;
                 } else if *cur_key > k_level {
                     for v in n_prefix.iter().take(n_prefix.len()).skip(i + 1) {
                         key_tracker.push(*v);
                     }
-                    return Ok(cmp::Ordering::Greater);
+                    return cmp::Ordering::Greater;
                 }
 
                 *level += 1;
             }
         }
-        Ok(cmp::Ordering::Equal)
+        cmp::Ordering::Equal
     }
 
     fn check_prefix_equals(
